@@ -20,7 +20,7 @@ struct SPVM_field {
   int32_t flag;
   int32_t runtime_type;
   int32_t is_captured;
-  int32_t byte_offset;
+  int32_t offset;
   int32_t has_setter;
   int32_t has_getter;
 };
