@@ -76,15 +76,15 @@ enum {
   SPVM_OP_C_ID_POST_INC,
   SPVM_OP_C_ID_PRE_DEC,
   SPVM_OP_C_ID_POST_DEC,
-  SPVM_OP_C_ID_COMPLEMENT,
+  SPVM_OP_C_ID_COND_NOT,
   SPVM_OP_C_ID_NEGATE,
   SPVM_OP_C_ID_PLUS,
-  SPVM_OP_C_ID_EQ,
-  SPVM_OP_C_ID_NE,
-  SPVM_OP_C_ID_LT,
-  SPVM_OP_C_ID_LE,
-  SPVM_OP_C_ID_GT,
-  SPVM_OP_C_ID_GE,
+  SPVM_OP_C_ID_NUMERIC_EQ,
+  SPVM_OP_C_ID_NUMERIC_NE,
+  SPVM_OP_C_ID_NUMERIC_LT,
+  SPVM_OP_C_ID_NUMERIC_LE,
+  SPVM_OP_C_ID_NUMERIC_GT,
+  SPVM_OP_C_ID_NUMERIC_GE,
   SPVM_OP_C_ID_ADD,
   SPVM_OP_C_ID_SUBTRACT,
   SPVM_OP_C_ID_MULTIPLY,
@@ -97,9 +97,9 @@ enum {
   SPVM_OP_C_ID_LEFT_SHIFT,
   SPVM_OP_C_ID_RIGHT_SHIFT,
   SPVM_OP_C_ID_RIGHT_SHIFT_UNSIGNED,
-  SPVM_OP_C_ID_AND,
-  SPVM_OP_C_ID_OR,
-  SPVM_OP_C_ID_NOT,
+  SPVM_OP_C_ID_COND_AND,
+  SPVM_OP_C_ID_COND_OR,
+  SPVM_OP_C_ID_COND_COMPELEMNT,
   SPVM_OP_C_ID_ARRAY_ACCESS,
   SPVM_OP_C_ID_ASSIGN,
   SPVM_OP_C_ID_CALL_SUB,
